import { Options } from './options.js';
import { TemplateService } from './template-service.js';
import { ProviderService } from './provider-service.js';
import { ProviderType } from './provider-type.js';
export class Airhorn {
    constructor(options) {
        this.options = new Options();
        this._templateService = new TemplateService();
        this._providerService = new ProviderService();
        if (options) {
            this.options = new Options(options);
            this._templateService = new TemplateService(options);
            this._providerService = new ProviderService(options);
        }
    }
    get templates() {
        return this._templateService;
    }
    get providers() {
        return this._providerService;
    }
    /* eslint max-params: [2, 6] */
    async send(to, from, templateName, providerType, data, languageCode) {
        let result = false;
        const template = this._templateService.getTemplate(templateName);
        if (template) {
            const providers = this._providerService.getProviderByType(providerType);
            if (providers.length > 0) {
                const message = await template.render(providerType, data, languageCode);
                if (message) {
                    const rand = Math.floor(Math.random() * providers.length);
                    const provider = providers[rand];
                    if (providerType === ProviderType.SMTP) {
                        const subject = template.getProperty(providerType, 'subject');
                        result = await provider.send(to, from, message, subject);
                    }
                    else {
                        result = await provider.send(to, from, message);
                    }
                }
            }
        }
        return result;
    }
    async sendSMTP(to, from, templateName, data, languageCode) {
        return this.send(to, from, templateName, ProviderType.SMTP, data, languageCode);
    }
    async sendSMS(to, from, templateName, data, languageCode) {
        return this.send(to, from, templateName, ProviderType.SMS, data, languageCode);
    }
    async sendWebhook(to, from, templateName, data, languageCode) {
        return this.send(to, from, templateName, ProviderType.WEBHOOK, data, languageCode);
    }
    async sendMobilePush(to, from, templateName, data, languageCode) {
        return this.send(to, from, templateName, ProviderType.MOBILE_PUSH, data, languageCode);
    }
}
export { ProviderType } from './provider-type.js';
//# sourceMappingURL=airhorn.js.map